﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;

namespace ThreadWatchDogCon
{
    public static class NativeMethods
    {
        [Flags]
        public enum ThreadAccess : int
        {
            TERMINATE = (0x0001),
            SUSPEND_RESUME = (0x0002),
            GET_CONTEXT = (0x0008),
            SET_CONTEXT = (0x0010),
            SET_INFORMATION = (0x0020),
            QUERY_INFORMATION = (0x0040),
            SET_THREAD_TOKEN = (0x0080),
            IMPERSONATE = (0x0100),
            DIRECT_IMPERSONATION = (0x0200)
        }

        [DllImport("kernel32.dll")]
        private static extern IntPtr OpenThread(ThreadAccess dwDesiredAccess, bool bInheritHandle, uint dwThreadId);

        [DllImport("kernel32.dll")]
        private static extern uint SuspendThread(IntPtr hThread);

        [DllImport("kernel32.dll")]
        private static extern int ResumeThread(IntPtr hThread);

        public static void SuspendProcess(int processId)
        {
            Process proc = Process.GetProcessById(processId);

            if (proc.ProcessName == string.Empty)
            {
                return;
            }

            foreach (ProcessThread thread in proc.Threads)
            {
                NativeMethods.SuspendThread(thread.Id);
            }
        }

        public static void SuspendThread(int threadId)
        {
            IntPtr pOpenThread = NativeMethods.OpenThread(ThreadAccess.SUSPEND_RESUME, false, (uint)threadId);

            if (pOpenThread == IntPtr.Zero)
            {
                return;
            }

            NativeMethods.SuspendThread(pOpenThread);
        }

        public static void ResumeProcess(int processId)
        {
            Process proc = Process.GetProcessById(processId);

            if (proc.ProcessName == string.Empty)
            {
                return;
            }

            foreach (ProcessThread thread in proc.Threads)
            {
                NativeMethods.ResumeThread(thread.Id);
            }
        }

        public static void ResumeThread(int threadId)
        {
            IntPtr pOpenThread = NativeMethods.OpenThread(ThreadAccess.SUSPEND_RESUME, false, (uint)threadId);

            if (pOpenThread == IntPtr.Zero)
            {
                return;
            }

            NativeMethods.ResumeThread(pOpenThread);
        }
    }
}
